﻿using System;
using System.Collections.Generic; 

namespace MultiSurfaceChartApp
{
    public class DataViewModel : ObservableModel
    {
        public DataViewModel()
        {
            _SelectedData = new List<DataPoint3D>();
            _SurfaceData = new List<DataPoint3D>();

            GenerateSampleData();

            SelectedDataIndex = 75;
        }

        #region Properties
        private List<DataPoint3D> _SurfaceData;
        /// <summary> Gets or sets SurfaceData </summary>
        public List<DataPoint3D> SurfaceData
        {
            get { return _SurfaceData; }
            set
            {
                if (_SurfaceData == value)
                    return; _SurfaceData = value;
                OnPropertyChanged("SurfaceData");
            }
        }

        private List<DataPoint3D> _SelectedData;
        /// <summary> Gets or sets SelectedData </summary>
        public List<DataPoint3D> SelectedData
        {
            get { return _SelectedData; }
            private set
            {
                if (_SelectedData == value)
                    return; _SelectedData = value;
                OnPropertyChanged("SelectedData");
            }
        }

        private int _SelectedDataIndex ;
        /// <summary> Gets or sets SelectedDataIndex </summary>
        public int SelectedDataIndex
        {
            get { return _SelectedDataIndex; }
            set
            {
                _SelectedDataIndex = value;
                OnPropertyChanged("SelectedDataIndex");
                UpdateSelection();
            }
        }
        
        private double _SelectedDataSizeX = 10;
        /// <summary> Gets or sets SelectedDataSizeX </summary>
        public double SelectedDataSizeX
        {
            get { return _SelectedDataSizeX; }
            set
            {
                _SelectedDataSizeX = value;
                OnPropertyChanged("SelectedDataSizeX");
                UpdateSelection();
            }
        } 

        private double _SelectedDataSizeY = 20;
        /// <summary> Gets or sets SelectedDataSize </summary>
        public double SelectedDataSizeY
        {
            get { return _SelectedDataSizeY; }
            set
            {
                _SelectedDataSizeY = value;
                OnPropertyChanged("SelectedDataSizeY");
                UpdateSelection();
            }
        } 
        #endregion

        private void UpdateSelection()
        {
            this.SelectedData.Clear();

            if (SurfaceData == null) return;
            if (SurfaceData.Count == 0) return;  
            if (SurfaceData.Count <= SelectedDataIndex) return;
            if (SelectedDataIndex < 0) return;
            if (SelectedDataSizeX < 0) return;
            if (SelectedDataSizeY < 0) return;

            var offsetX = this.SelectedDataSizeX / 2.0;
            var offsetY = this.SelectedDataSizeY / 2.0;
            var center = this.SurfaceData[SelectedDataIndex];
            var x = center.X;
            var y = center.Y;
            // ensure that selected points have slightly higher Z value than surface data
            var z = center.Z + (center.Z * 0.00001); 
            var selectedRegion = new List<DataPoint3D>();
            selectedRegion.Add(new DataPoint3D(x, y, center.Z));
            // TODO add more points to change shape of the selected region 
            selectedRegion.Add(new DataPoint3D(x - offsetX, y - offsetY, z));
            selectedRegion.Add(new DataPoint3D(x - offsetX, y + offsetY, z));
            selectedRegion.Add(new DataPoint3D(x + offsetX, y + offsetY, z));
            selectedRegion.Add(new DataPoint3D(x + offsetX, y - offsetY, z));

            this.SelectedData = selectedRegion;
        } 
         
        private void GenerateSampleData()
        {  
            for (int x = 0; x < 100; x+=10)
            {
                for (int y= 0; y < 100; y+=10)
                { 
                    _SurfaceData.Add(new DataPoint3D(x, y, 50));
                }
            } 
        }
    }

}
